@@ -1,18 +1,16 @@
 # SPDX-FileCopyrightText: 2017 Free Software Foundation Europe e.V. <https://fsfe.org>
-# SPDX-FileCopyrightText: © 2020 Liferay, Inc. <https://liferay.com>
-# SPDX-FileCopyrightText: 2022 Nico Rikken <nico.rikken@fsfe.org>
-# SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
 # SPDX-FileCopyrightText: 2022 Carmen Bianca Bakker <carmenbianca@fsfe.org>
+# SPDX-FileCopyrightText: 2022 Florian Snow <florian@familysnow.net>
+# SPDX-FileCopyrightText: 2022 Nico Rikken <nico.rikken@fsfe.org>
 # SPDX-FileCopyrightText: 2022 Pietro Albini <pietro.albini@ferrous-systems.com>
+# SPDX-FileCopyrightText: 2024 Rivos Inc.
+# SPDX-FileCopyrightText: © 2020 Liferay, Inc. <https://liferay.com>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 """Tests for reuse._util"""
 
-# pylint: disable=protected-access,invalid-name
-
 import os
-import shutil
 from argparse import ArgumentTypeError
 from inspect import cleandoc
 from io import BytesIO
@@ -20,32 +18,11 @@ from pathlib import Path
 
 import pytest
 from boolean.boolean import ParseError
-from debian.copyright import Copyright
-from debian.copyright import Error as DebianError
-from license_expression import LicenseSymbol
+from conftest import no_root, posix
 
 from reuse import _util
 from reuse._util import _LICENSING
 
-try:
-    import pwd
-
-    is_root = pwd.getpwuid(os.getuid()).pw_name == "root"
-    is_posix = True
-except ImportError:
-    is_root = False
-    is_posix = False
-
-
-git = pytest.mark.skipif(not _util.GIT_EXE, reason="requires git")
-no_root = pytest.mark.xfail(is_root, reason="fails when user is root")
-posix = pytest.mark.skipif(not is_posix, reason="Windows not supported")
-
-
-TESTS_DIRECTORY = Path(__file__).parent.resolve()
-RESOURCES_DIRECTORY = TESTS_DIRECTORY / "resources"
-
-
 # REUSE-IgnoreStart
 
 
@@ -392,60 +369,6 @@ def test_filter_ignore_block_with_multiple_ignore_blocks():
     assert result == expected
 
 
-def test_parse_dep5_simple(fake_repository):
-    """No error if everything is good."""
-    result = _util._parse_dep5(fake_repository / ".reuse/dep5")
-    assert result.__class__ == Copyright
-
-
-def test_parse_dep5_not_exists(empty_directory):
-    """Raise FileNotFoundError if .reuse/dep5 doesn't exist."""
-    with pytest.raises(FileNotFoundError):
-        _util._parse_dep5(empty_directory / "foo")
-
-
-def test_parse_dep5_unicode_decode_error(fake_repository):
-    """Raise UnicodeDecodeError if file can't be decoded as utf-8."""
-    shutil.copy(RESOURCES_DIRECTORY / "fsfe.png", fake_repository / "fsfe.png")
-    with pytest.raises(UnicodeDecodeError):
-        _util._parse_dep5(fake_repository / "fsfe.png")
-
-
-def test_parse_dep5_parse_error(empty_directory):
-    """Raise DebianError on parse error."""
-    (empty_directory / "foo").write_text("foo")
-    with pytest.raises(DebianError):
-        _util._parse_dep5(empty_directory / "foo")
-
-
-def test_parse_dep5_double_copyright_parse_error(empty_directory):
-    """Raise DebianError on double Copyright lines."""
-    (empty_directory / "foo").write_text(
-        cleandoc(
-            """
-            Format: something
-            Upstream-Name: example
-            Upstream-Contact: Jane Doe
-            Source: https://example.com
-
-            Files: *
-            Copyright: Jane Doe
-            Copyright: John Doe
-            License: MIT
-            """
-        )
-    )
-    with pytest.raises(DebianError):
-        _util._parse_dep5(empty_directory / "foo")
-
-
-def test_copyright_from_dep5(dep5_copyright):
-    """Verify that the glob in the dep5 file is matched."""
-    result = _util._copyright_from_dep5("doc/foo.rst", dep5_copyright)
-    assert LicenseSymbol("CC0-1.0") in result.spdx_expressions
-    assert "2017 Jane Doe" in result.copyright_lines
-
-
 def test_make_copyright_line_simple():
     """Given a simple statement, make it a copyright line."""
     assert _util.make_copyright_line("hello") == "SPDX-FileCopyrightText: hello"
@@ -459,64 +382,64 @@ def test_make_copyright_line_year():
     )
 
 
-def test_make_copyright_line_style_spdx():
-    """Given a simple statement and style, make it a copyright line."""
-    statement = _util.make_copyright_line("hello", copyright_style="spdx")
+def test_make_copyright_line_prefix_spdx():
+    """Given a simple statement and prefix, make it a copyright line."""
+    statement = _util.make_copyright_line("hello", copyright_prefix="spdx")
     assert statement == "SPDX-FileCopyrightText: hello"
 
 
-def test_make_copyright_line_style_spdx_year():
-    """Given a simple statement, style and a year, make it a copyright line."""
+def test_make_copyright_line_prefix_spdx_year():
+    """Given a simple statement, prefix and a year, make it a copyright line."""
     statement = _util.make_copyright_line(
-        "hello", year=2019, copyright_style="spdx"
+        "hello", year=2019, copyright_prefix="spdx"
     )
     assert statement == "SPDX-FileCopyrightText: 2019 hello"
 
 
-def test_make_copyright_line_style_spdx_c_year():
-    """Given a simple statement, style and a year, make it a copyright line."""
+def test_make_copyright_line_prefix_spdx_c_year():
+    """Given a simple statement, prefix and a year, make it a copyright line."""
     statement = _util.make_copyright_line(
-        "hello", year=2019, copyright_style="spdx-c"
+        "hello", year=2019, copyright_prefix="spdx-c"
     )
     assert statement == "SPDX-FileCopyrightText: (C) 2019 hello"
 
 
-def test_make_copyright_line_style_spdx_symbol_year():
-    """Given a simple statement, style and a year, make it a copyright line."""
+def test_make_copyright_line_prefix_spdx_symbol_year():
+    """Given a simple statement, prefix and a year, make it a copyright line."""
     statement = _util.make_copyright_line(
-        "hello", year=2019, copyright_style="spdx-symbol"
+        "hello", year=2019, copyright_prefix="spdx-symbol"
     )
     assert statement == "SPDX-FileCopyrightText: © 2019 hello"
 
 
-def test_make_copyright_line_style_string_year():
-    """Given a simple statement, style and a year, make it a copyright line."""
+def test_make_copyright_line_prefix_string_year():
+    """Given a simple statement, prefix and a year, make it a copyright line."""
     statement = _util.make_copyright_line(
-        "hello", year=2019, copyright_style="string"
+        "hello", year=2019, copyright_prefix="string"
     )
     assert statement == "Copyright 2019 hello"
 
 
-def test_make_copyright_line_style_string_c_year():
-    """Given a simple statement, style and a year, make it a copyright line."""
+def test_make_copyright_line_prefix_string_c_year():
+    """Given a simple statement, prefix and a year, make it a copyright line."""
     statement = _util.make_copyright_line(
-        "hello", year=2019, copyright_style="string-c"
+        "hello", year=2019, copyright_prefix="string-c"
     )
     assert statement == "Copyright (C) 2019 hello"
 
 
-def test_make_copyright_line_style_string_symbol_year():
-    """Given a simple statement, style and a year, make it a copyright line."""
+def test_make_copyright_line_prefix_string_symbol_year():
+    """Given a simple statement, prefix and a year, make it a copyright line."""
     statement = _util.make_copyright_line(
-        "hello", year=2019, copyright_style="string-symbol"
+        "hello", year=2019, copyright_prefix="string-symbol"
     )
     assert statement == "Copyright © 2019 hello"
 
 
-def test_make_copyright_line_style_symbol_year():
-    """Given a simple statement, style and a year, make it a copyright line."""
+def test_make_copyright_line_prefix_symbol_year():
+    """Given a simple statement, prefix and a year, make it a copyright line."""
     statement = _util.make_copyright_line(
-        "hello", year=2019, copyright_style="symbol"
+        "hello", year=2019, copyright_prefix="symbol"
     )
     assert statement == "© 2019 hello"
 
